@@ -1,7 +1,7 @@
 from contextlib import contextmanager
 from dataclasses import dataclass
 from enum import Enum, auto
-from typing import Any, Dict, List, Optional, Set, Tuple
+from typing import Any, Dict, Iterator, List, Optional, Set, Tuple
 
 from . import ast
 from .renderer import INDENT
@@ -12,39 +12,6 @@ def parse(text: str) -> ast.Changelog:
     return Parser(text, tokens)()
 
 
-class ParseError(Exception):
-    def __init__(
-        self,
-        msg: str,
-        source: str,
-        token: "Token",
-        back: int = 0,
-        forward: int = 0,
-        hint: str = "",
-    ):
-        msg = f"{msg} {annotate(source, token, back, forward)}"
-        if hint:
-            msg += f"\n\nHint: {hint}"
-        super().__init__(msg)
-
-
-def annotate(source: str, token: "Token", back: int = 0, forward: int = 0) -> str:
-    lines = source.splitlines()
-    row = min(token.row, len(lines) - 1)
-    prev, line = lines[row - 1 : row + 1] if row > 0 else ("", lines[row])
-
-    if not (back or forward):
-        forward = len(token.text)
-
-    col_start = token.col - back
-    col_end = token.col + forward
-    col = max(0, col_start)
-
-    arrows = " " * col_start + "^" * (col_end - col_start)
-    details = (f"  {prev}\n" if prev else "") + f"  {line}\n  {arrows}"
-    return f"at line {row + 1}, column {col + 1}:\n\n{details}"
-
-
 class TokenType(Enum):
     HASH = auto()
     DASH = auto()
@@ -89,7 +56,7 @@ class Tokenizer:
         except IndexError:
             return "\0"
 
-    def add_token(self, typ: TokenType, text: str = "", parsed: Any = None):
+    def add_token(self, typ: TokenType, text: str = "", parsed: Any = None) -> None:
         col = self.col - len(self.matched)
         token = Token(typ, self.row, col, text or self.matched, parsed)
         self.tokens.append(token)
@@ -110,7 +77,7 @@ def scan_token(t: Tokenizer) -> None:
     elif char == "-":
         t.add_token(TokenType.DASH)
     elif char == "\n":
-        t.add_token(TokenType.NEWLINE, char)
+        t.add_token(TokenType.NEWLINE)
         t.row += 1
         t.col = 0
     elif char.isspace():
@@ -125,6 +92,10 @@ def scan_token(t: Tokenizer) -> None:
             t.add_token(TokenType.TEXT, text)
 
 
+class ParseError(Exception):
+    pass
+
+
 class Parser:
     def __init__(self, source: str, tokens: List[Token]):
         self.source = source
@@ -139,17 +110,16 @@ class Parser:
         pass
 
     @contextmanager
-    def checkpoint(self):
+    def checkpoint(self) -> Iterator[None]:
         saved = self.current
         try:
             yield
         except self.Rollback:
             self.current = saved
 
-    def advance(self, n: int = 1):
+    def advance(self, n: int = 1) -> None:
         if self.has_more:
             self.current += n
-        return self.peek()
 
     def check(self, typ: TokenType) -> bool:
         return self.has_more and self.peek().typ is typ
@@ -187,7 +157,10 @@ class Parser:
         hint: str = "",
     ) -> ParseError:
         token = token or self.peek()
-        return ParseError(msg, self.source, token, back, forward, hint)
+        msg += " " + annotate(self.source, token, back, forward)
+        if hint:
+            msg += f"\n\nHint: {hint}"
+        return ParseError(msg)
 
     def __call__(self) -> ast.Changelog:
         return changelog(self)
@@ -304,3 +277,20 @@ def changes(p: Parser) -> Tuple[ast.ChangeType, ast.Changes]:
 def skip_newlines(p: Parser) -> None:
     while p.match({TokenType.NEWLINE}):
         pass
+
+
+def annotate(source: str, token: Token, back: int = 0, forward: int = 0) -> str:
+    lines = source.splitlines()
+    row = min(token.row, len(lines) - 1)
+    prev, line = lines[row - 1 : row + 1] if row > 0 else ("", lines[row])
+
+    if not (back or forward):
+        forward = len(token.text)
+
+    col_start = token.col - back
+    col_end = token.col + forward
+    col = max(0, col_start)
+
+    arrows = " " * col_start + "^" * (col_end - col_start)
+    details = (f"  {prev}\n" if prev else "") + f"  {line}\n  {arrows}"
+    return f"at line {row + 1}, column {col + 1}:\n\n{details}"
